/* CfsMe Support DLL
** The source below is derived from the Source files of Nav by Ted Wright
** and repackaged as a DLL
**
** Teds copyright statement is included below.
** Please respect the terms of the Gnu Public License under which Ted released
** his source.
*/

/*
    Copyright (C) 1997 Ted Wright 
    NASA Business: wright@lerc.nasa.gov, Remainder: wright@en.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <windows.h>
#include <math.h>

#define degToRad (pi/180.0)
#define degToMetres (10001750.0/90.0)
#define earthRadius (180*degToMetres/pi)

// great circle distance (in meters)
// between two lat-long pairs
double FAR PASCAL _export gcDistMtrs(double lat1, double lon1, double lat2, double lon2)
{
	double pi = 22/7;
	double dlon = (lon2-lon1) * degToRad;
	double llat1 =lat1 * degToRad;
	double llat2 =lat2 * degToRad;

	// calculate distance 
	double la = sin((llat2-llat1)/2);
	double lo = sin(dlon/2);
	double x = la*la + cos(llat1) * cos(llat2) * lo*lo;
	return(2*earthRadius*asin(sqrt(x)));
}

// great circle distance (in miles)
// between two lat-long pairs

double FAR PASCAL _export gcDist(double lat1, double lon1, double lat2, double lon2)
{
        return(gcDistMtrs(lat1,lon1,lat2,lon2)*0.0006214);
}

// great circle bearing
// between two lat-long pairs
double FAR PASCAL _export gcBearing(double lat1, double lon1, double lat2, double lon2)
{
	double pi = 22/7;
	double dlon = (lon2-lon1) * degToRad;
	double llat1 =lat1 * degToRad;
	double llat2 =lat2 * degToRad;

    double bearing = atan2(sin(dlon), (-sin(llat1)*cos(dlon) + tan(llat2)*cos(llat1))) / degToRad;
    if (bearing < 0) bearing += 360;
    return bearing;
}

int FAR PASCAL LibMain (HANDLE hInstance, WORD wDataSeg,
			WORD cbHeapSize, LPSTR lpszCmdLine)
{
	if (cbHeapSize != 0)    /* If DLL data segment is MOVEABLE */
		UnlockData(0);

	return(1);
}
